package cn.kdan.cloud.pdf.office.api.payment.feign;

import cn.kdan.cloud.pdf.office.api.payment.dto.CreateSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateSubscriptionManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.feign.hystrix.SubscriptionHystrix;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionSchemeResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionSchemeVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;


@Component
@FeignClient(value = CommonConstant.PAYMENT_PROVIDER_NAME + "/subscription", fallback = SubscriptionHystrix.class)
public interface SubscriptionApi {
    /**
     * 创建订阅信息
     *
     * @param subscriptionInfo subscriptionInfo
     * @return 订阅id
     */
    @PostMapping("/createSubscription")
    ResultMap<String> createSubscription(@Validated @RequestBody CreateSubscriptionDTO subscriptionInfo);

    /**
     * 通过用户id和产品id 获取最新的订阅信息
     *
     * @param userId   userId
     * @param platform platform
     * @return Subscriptions
     */
    @GetMapping("/getSubscriptions")
    ResultMap<SubscriptionsVO> getSubscriptions(@Validated @NotNull(message = "userId 不能为空") @RequestParam("userId") String userId,
                                                @Validated @NotNull(message = "platform 不能为空") @RequestParam("platform") Integer platform);

    /**
     * 根据产品id获取订阅信息
     * @param userId 用户id
     * @param productId 产品id
     * @return SubscriptionsVO
     */
    @GetMapping("/getSubscriptionsByProductId")
    ResultMap<SubscriptionsVO> getSubscriptionsByProductId(@Validated @NotNull(message = "userId 不能为空") @RequestParam("userId") String userId,
                                                @Validated @NotNull(message = "productId 不能为空") @RequestParam("productId") String productId);


    /**
     * 查询最新一条实际支付的订阅信息（排除赠送的订阅）
     * @param userId 用户id
     * @param productId 产品id
     * @return SubscriptionsVO
     */
    @GetMapping("/getLatestSubscriptionForActualPayment")
    ResultMap<SubscriptionsVO> getLatestSubscriptionForActualPayment(@Validated @NotNull(message = "userId 不能为空") @RequestParam("userId") String userId,
                                                           @Validated @NotNull(message = "productId 不能为空") @RequestParam("productId") String productId);


    /**
     * 手动创建订阅信息
     *
     * @param subscriptionManual subscriptionManual
     * @return 订阅id
     */
    @PostMapping("/giveSubscriptionManual")
    ResultMap<String> giveSubscriptionManual(@Validated @RequestBody CreateSubscriptionManualDTO subscriptionManual);


    /**
     * 查询当前用户的订阅计划和支付订单
     *
     * @param userId   userId
     * @return SubscriptionSchemeVO
     */
    @GetMapping("/getSubscriptionSchemeInfo")
    ResultMap<Map<String,SubscriptionSchemeVO>> getSubscriptionSchemeInfo(@RequestParam("userId") String userId,
                                                                          @RequestParam("appId") String appId);


    @GetMapping("/getSubscriptionsByUserIdAndProductId")
    ResultMap<List<SubscriptionsVO>> getSubscriptionsByUserIdAndProductId(@RequestParam("userId") String userId,
                                                                          @RequestParam("productId") String productId);

    /**
     * 获取某个产品的下一次实际订阅扣款时间
     * @param userId 用户id
     * @param productId 产品id
     * @return LocalDateTime
     */
    @GetMapping("/getNextPayTime")
    ResultMap<LocalDateTime> getNextPayTime(@RequestParam("userId") String userId,
                                            @RequestParam("productId") String productId);

    /**
     * 判断是否可以送试用
     * @param userId 用户id
     * @return Boolean
     */
    @GetMapping("/judgeTrail")
    ResultMap<Boolean> judgeTrail(@RequestParam("userId") String userId);
    @GetMapping("/judgeAiDiscount")
    ResultMap<Boolean> judgeAiDiscount(@RequestParam("userId") String userId);
}
