package cn.kdan.cloud.pdf.office.payment.client;

import cn.kdan.cloud.pdf.office.api.payment.bo.CreateSubscriptionsResultPaypalBO;
import cn.kdan.cloud.pdf.office.api.payment.bo.GeneratePayLinkPaddleBO;
import cn.kdan.cloud.pdf.office.api.payment.bo.GeneratePayLinkResultPaddleBO;
import cn.kdan.cloud.pdf.office.api.payment.constant.PaddleAPIConstant;
import cn.kdan.cloud.pdf.office.api.payment.paddle.PaddleResult;
import cn.kdan.cloud.pdf.office.api.payment.paddle.SubscriptionWithUser;
import cn.kdan.cloud.pdf.office.api.payment.paddle.UpdateSubscription;
import cn.kdan.cloud.pdf.office.api.payment.paddle.UpdateSubscriptionResult;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import cn.kdan.cloud.pdf.office.payment.interceptor.PaddleClientHttpRequestInterceptor;
import cn.kdan.cloud.pdf.office.payment.properties.PaddleProperties;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.google.common.collect.Iterables;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.ObjectUtils;
import org.springframework.web.client.DefaultResponseErrorHandler;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author ComPDFKit-WPH 2023/1/16
 * <p>
 * PaddleClient
 */
@Component
@Slf4j
public class PaddleClient {

    private final RestTemplate restTemplate = new RestTemplate();

    private static String PADDLE_CHECKOUT_HOST;

    private static String PADDLE_VENDORS_HOST;

    private final PaddleProperties properties;

    public PaddleClient(PaddleProperties properties, PaddleClientHttpRequestInterceptor paddleClientHttpRequestInterceptor) {
        this.properties = properties;
        PADDLE_CHECKOUT_HOST = properties.getCheckoutHost();
        PADDLE_VENDORS_HOST = properties.getVendorsHost();
        SimpleClientHttpRequestFactory factory = new SimpleClientHttpRequestFactory();
        factory.setReadTimeout(properties.getReadTimeOut());
        factory.setConnectTimeout(properties.getConnectTimeout());
        restTemplate.setRequestFactory(factory);
        restTemplate.setInterceptors(Collections.singletonList(paddleClientHttpRequestInterceptor));
        restTemplate.setErrorHandler(new DefaultResponseErrorHandler() {
            @Override
            public void handleError(@NotNull ClientHttpResponse clientHttpResponse) throws IOException {
                // 只要重写此方法，不去抛出HttpClientErrorException异常即可
                // HttpStatus statusCode = clientHttpResponse.getStatusCode();
            }
        });
    }

    /**
     * 创建订阅 生成支付链接
     *
     * @param createSubscriptions createSubscriptions
     * @return CreateSubscriptionsResultBO
     */
    public GeneratePayLinkResultPaddleBO createSubscriptions(GeneratePayLinkPaddleBO createSubscriptions) {
        MultiValueMap<String, Object> requestParamMap = getValueMap(createSubscriptions);
        Map<String, Object> stringObjectMap = JSON.parseObject(JsonUtils.getJsonString(createSubscriptions),
                new TypeReference<Map<String, Object>>() {
                });
        Iterables.removeIf(stringObjectMap.values(), Objects::isNull);
        requestParamMap.setAll(stringObjectMap);
        String[] prices = createSubscriptions.getPrices();
        if (!ObjectUtils.isEmpty(prices)) {
            requestParamMap.remove("prices");
            for (int i = 0; i < prices.length; i++) {
                requestParamMap.set("prices[" + i + "]:", prices[i]);
            }
        }
        log.info("prama:{}", requestParamMap);
        String resultJson = restTemplate.exchange(
                PADDLE_VENDORS_HOST + PaddleAPIConstant.CREAT_PAY_LINK,
                HttpMethod.POST,
                new HttpEntity<>(requestParamMap, getWwwFormHttpHeaders()),
                String.class).getBody();
        PaddleResult<GeneratePayLinkResultPaddleBO> paddleResult = JSON.parseObject(resultJson,
                new TypeReference<PaddleResult<GeneratePayLinkResultPaddleBO>>() {
                });

        if (!paddleResult.isSuccess()) {
            log.error("paddle 生成支付链接调用失败\n：{},\n参数：{}", paddleResult.getError(), createSubscriptions);
            throw new BackendRuntimeException(paddleResult.getError().getMessage());
        }
        return paddleResult.getResponse();
    }

    /**
     * paddle 取消续订
     *
     * @param thirdSubscriptionId paddle subscriptionId
     * @return paddle subscriptionId
     */
    public String cancelSubscription(String thirdSubscriptionId) {
        log.info("paddle 取消续订:{}", thirdSubscriptionId);
        MultiValueMap<String, Object> requestParamMap = getValueMap(null);
        requestParamMap.set("subscription_id", thirdSubscriptionId);
        String resultJson = restTemplate.exchange(
                PADDLE_VENDORS_HOST + PaddleAPIConstant.CANCEL_SUBSCRIPTION,
                HttpMethod.POST,
                new HttpEntity<>(requestParamMap, getWwwFormHttpHeaders()),
                String.class).getBody();
        PaddleResult<Void> paddleResult = JSON.parseObject(resultJson,
                new TypeReference<PaddleResult<Void>>() {
                });

        if (!paddleResult.isSuccess()) {
            log.error("paddle 取消续订调用失败\n：{},\n参数：{}", paddleResult.getError(), thirdSubscriptionId);
            throw new BackendRuntimeException(paddleResult.getError().getMessage());
        }
        return thirdSubscriptionId;
    }

    /**
     * 更新订阅
     *
     * @param updateSubscription updateSubscription
     * @return UpdateSubscriptionResult
     */
    public UpdateSubscriptionResult updateSubscription(UpdateSubscription updateSubscription) {
        log.info("paddle 更新订阅:{}", updateSubscription);
        MultiValueMap<String, Object> requestParamMap = getValueMap(updateSubscription);
        String resultJson = restTemplate.exchange(
                PADDLE_VENDORS_HOST + PaddleAPIConstant.UPDATE_SUBSCRIPTION,
                HttpMethod.POST,
                new HttpEntity<>(requestParamMap, getWwwFormHttpHeaders()),
                String.class).getBody();
        PaddleResult<UpdateSubscriptionResult> paddleResult = JSON.parseObject(resultJson,
                new TypeReference<PaddleResult<UpdateSubscriptionResult>>() {
                });
        if (!paddleResult.isSuccess()) {
            log.error("paddle 生成支付链接调用失败\n：{},\n参数：{}", paddleResult.getError(), updateSubscription);
            throw new BackendRuntimeException(paddleResult.getError().getMessage());
        }
        return paddleResult.getResponse();
    }


    @NotNull
    private MultiValueMap<String, Object> getValueMap(Object o) {
        MultiValueMap<String, Object> requestParamMap = new LinkedMultiValueMap<>();
        requestParamMap.set(PaddleAPIConstant.VENDORID, properties.getVendorId());
        requestParamMap.set(PaddleAPIConstant.VENDORAUTHCODE, properties.getVendorAuthCode());
        if (!ObjectUtils.isEmpty(o)) {
            Map<String, Object> stringObjectMap = JSON.parseObject(JsonUtils.getJsonString(o),
                    new TypeReference<Map<String, Object>>() {
                    });
            Iterables.removeIf(stringObjectMap.values(), Objects::isNull);
            requestParamMap.setAll(stringObjectMap);
        }
        return requestParamMap;
    }


    /**
     * 获取订阅信息
     *
     * @param subscriptionId subscriptionId
     * @return CreateSubscriptionsResultBO
     */
    public List<SubscriptionWithUser> getSubscriptionInfo(@NotNull String subscriptionId) {
        log.info("paddle 查询订阅:{}", subscriptionId);
        MultiValueMap<String, Object> requestParamMap = getValueMap(null);
        requestParamMap.set("subscription_id", subscriptionId);
        requestParamMap.set("page", 1);
        requestParamMap.set("results_per_page", 1);
        requestParamMap.set("state", "active");
        String resultJson = restTemplate.exchange(
                PADDLE_VENDORS_HOST + PaddleAPIConstant.USER_SUBSCRIPTION,
                HttpMethod.POST,
                new HttpEntity<>(requestParamMap, getWwwFormHttpHeaders()),
                String.class).getBody();
        PaddleResult<List<SubscriptionWithUser>> paddleResult = JSON.parseObject(resultJson,
                new TypeReference<PaddleResult<List<SubscriptionWithUser>>>() {
                });

        if (!paddleResult.isSuccess()) {
            log.error("paddle 取消续订调用失败\n：{},\n参数：{}", paddleResult.getError(), subscriptionId);
            throw new BackendRuntimeException(paddleResult.getError().getMessage());
        }
        return paddleResult.getResponse();
    }


    @NotNull
    private HttpHeaders getWwwFormHttpHeaders() {
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        return httpHeaders;
    }

}
