package cn.kdan.cloud.pdf.office.payment.listener;

import cn.kdan.cloud.pdf.office.api.account.dto.UpdateUserForOrderDTO;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.payment.constant.OrderConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.AppStoreOrderSucceedDTO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.api.payment.constant.SubscriptionConstant;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.common.constant.RabbitMqConstant;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PayTypeEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.payment.service.AppStoreService;
import cn.kdan.cloud.pdf.office.payment.service.OrderGiftLogService;
import cn.kdan.cloud.pdf.office.payment.service.OrderService;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import cn.kdan.cloud.pdf.office.api.account.utils.SubscriptionUtil;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.rabbitmq.client.Channel;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;

/**
 * @author ComPDFKit-WPH 2023/1/11
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class PaymentListener {

    private final OrderService ordersService;

    private final SubscriptionsService subscriptionsService;

    private final UserApi userApi;

    private final StringRedisTemplate redisTemplate;

    private final AppStoreService appStoreService;
    private final OrderGiftLogService orderGiftLogService;

    private final RabbitTemplate rabbitTemplate;

    /**
     * 关闭订单 监听
     *
     * @param message message
     * @param channel channel
     * @param orderId orderId
     */
    @RabbitListener(queues = RabbitMqConstant.PAYMENT_ORDER_DELAY_QUEUE)
    public void paymentOrderDelayQueue(Message message, Channel channel, String orderId) {
        try {
            log.info("关闭订单 监听内容：{}", orderId);
            ordersService.closeOrder(orderId);
            log.info("订单关闭成功 :{}", orderId);
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
        } catch (Exception e) {
            log.error("消息消费失败:" + e.getMessage(), e);
        }
    }

    /**
     * 订阅过期 监听
     *
     * @param message        message
     * @param channel        channel
     * @param subscriptionId subscriptionId
     */
    @RabbitListener(queues = RabbitMqConstant.PAYMENT_SUBSCRIPTION_EXPIRED_DELAY_QUEUE)
    public void paymentSubscriptionExpiredDelayQueue(Message message, Channel channel, String subscriptionId) {
        try {
            log.info("订阅过期 监听内容：{}", subscriptionId);
            // 判断当前用户是否有有效的订阅模式
            SubscriptionsVO subscriptionsVO = subscriptionsService.getSubscriptionBySubscriptionId(subscriptionId);
            if(ObjectUtils.isEmpty(subscriptionsVO)){
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
                log.info("订阅不存在 处理成功 :{}", subscriptionId);
                return;
            }
            LocalDateTime now = LocalDateTime.now();
            UserInfoVO userInfoVO = userApi.getInfoById(subscriptionsVO.getUserId()).getResult();
            if(ObjectUtils.isEmpty(userInfoVO)){
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
                log.info("用户不存在 处理成功 :{}", subscriptionsVO.getUserId());
                return;
            }
            List<UserSubscriptionInfoVO> subscriptionInfoList =userInfoVO.getSubscriptionInfoList();
            UserSubscriptionInfoVO userSubscriptionInfoVO = SubscriptionUtil.getUserSubscriptionInfoByPlatform(subscriptionInfoList, subscriptionsVO.getPlatform());
            LocalDateTime endDate = MyDateUtils.dateToLocalDateTime(userSubscriptionInfoVO.getEndDate());
            LocalDateTime oldEndDate = MyDateUtils.dateToLocalDateTime(subscriptionsVO.getEndDate());
            // 修改订阅表订阅状态 过期
            log.info("订阅过期 当前时间 :{} 到期时间 :{}", now, oldEndDate);
            if (now.isBefore(oldEndDate)){
                SubscriptionUtil.subscriptionExpiredSendNews(rabbitTemplate,subscriptionId,oldEndDate);
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
                return;
            }
            log.info("订阅过期 处理成功 :{}", subscriptionId);
            subscriptionsService.updateSubscriptionStatus(subscriptionId, SubscriptionConstant.expired);
            if (now.isAfter(endDate)) {
                // 修改用户状态
                // 获取当前用户的订阅状态
                Integer userStatus = Integer.MIN_VALUE;
                if (!ObjectUtils.isEmpty(userSubscriptionInfoVO)) {
                    userStatus = userSubscriptionInfoVO.getStatus();
                    PDFOfficeUserSubscriptionStatusEnum userStatusEnum = PDFOfficeUserSubscriptionStatusEnum.getEnumByValue(userSubscriptionInfoVO.getStatus());
                    if (userStatusEnum == PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS){
                        userStatus = PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_EXPIRED.value();
                    }else if (userStatusEnum == PDFOfficeUserSubscriptionStatusEnum.TRIAL){
                        userStatus = PDFOfficeUserSubscriptionStatusEnum.TRIAL_EXPIRED.value();
                    }
                }
                // 修改用户状态
                UpdateUserForOrderDTO paySuccessDTO = new UpdateUserForOrderDTO();
                paySuccessDTO.setUserId(subscriptionsVO.getUserId());
                paySuccessDTO.setPlatform(subscriptionsVO.getPlatform());
                paySuccessDTO.setUserStatus(userStatus);
                paySuccessDTO.setPayTpe(PayTypeEnum.NO_AUTO.value());
                userApi.updateUserForOrder(paySuccessDTO);
                // 修改订阅表订阅状态 过期
                subscriptionsService.updateSubscriptionStatus(subscriptionId, SubscriptionConstant.expired);
            }
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
            log.info("订阅过期 处理成功 :{}", subscriptionId);
        } catch (Exception e) {
            log.error("消息消费失败:" + e.getMessage(), e);
            throw new BackendRuntimeException("消息消费失败");
        }
    }

    @RabbitListener(queues = RabbitMqConstant.ORDER_GIFT_DELAY_QUEUE)
    public void orderGiftDelayQueue(Message message, Channel channel, String orderGiftId) {
        try {
            log.info("赠送过期 监听内容：{}", orderGiftId);
            orderGiftLogService.expire(orderGiftId);
            log.info("赠送过期成功 :{}", orderGiftId);
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
        } catch (Exception e) {
            log.error("消息消费失败:" + e.getMessage(), e);
        }
    }

}
