package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.account.utils.SubscriptionUtil;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfo;
import cn.kdan.cloud.pdf.office.api.email.bo.EmailSendBO;
import cn.kdan.cloud.pdf.office.api.email.feign.EmailApi;
import cn.kdan.cloud.pdf.office.api.payment.constant.OrderConstant;
import cn.kdan.cloud.pdf.office.api.payment.constant.SubscriptionConstant;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.enums.EmailCodeTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.AccountTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionPayTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionStatusEnum;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.payment.entity.Order;
import cn.kdan.cloud.pdf.office.payment.entity.OrderGift;
import cn.kdan.cloud.pdf.office.payment.entity.Subscription;
import cn.kdan.cloud.pdf.office.payment.service.OrderGiftService;
import cn.kdan.cloud.pdf.office.payment.service.OrderService;
import cn.kdan.cloud.pdf.office.payment.service.PayCenterWebhookService;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import cn.kdan.cloud.pdf.office.payment.utils.TemplatesUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StreamUtils;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.time.ZoneId;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-WPH 2023/2/9
 */
@Service
@Slf4j
@RequiredArgsConstructor
@Transactional
public class PayCenterWebhookServiceImpl implements PayCenterWebhookService {

    private final UserApi userApi;

    private final UserSubscriptionInfoApi userSubscriptionInfoApi;

    private final ProductApi productApi;

    private final OrderService ordersService;

    private final SubscriptionsService subscriptionsService;

    private final RedissonClient redissonClient;

    private final RabbitTemplate rabbitTemplate;

    private final OrderGiftService orderGiftService;

    private final EmailApi emailApi;

    private static final RestTemplate restTemplate = new RestTemplate();

    @Value("${htmlToPdfUrl:http://139.196.160.101:3060/api/get-invoice}")
    private String htmlToPdfUrl;

    /**
     * 支付中台统一回调
     * @param  body  请求参数
     *                {
     *                 "event_type": "事件类型：1 支付成功(订阅激活成功)回调，2 支付失败回调，3 订阅扣款成功回调，4 订阅扣款失败回调，5 取消订阅回调",
     *                 "third_trade_id": "第三方订单号",
     *                 "pay_id": "第三方交易id",
     *                 "nextBillingTime": "订阅-下次扣款时间",
     *                 "order_num": "我们这边创建订单时传过去的自定义订单号",
     *                }
     * @return String 支付链接
     */
    @Override
    @Transactional
    public Boolean webhookMonitor(Map<String, String> body) {
        if (body == null || body.isEmpty()) {
            log.error("支付中台统一回调，请求参数为空");
            return false;
        }
        log.info("支付中台统一回调，请求参数：{}", body);
        String eventType = body.get("event_type");
        String thirdTradeId = body.get("third_trade_id");
        String payId = body.get("pay_id");
        String nextBillingTime = body.get("nextBillingTime");
        String orderNum = body.get("order_num");
        // 获取 RLock 对象
        String lockKey = eventType.concat(thirdTradeId);
        RLock rLock = redissonClient.getLock(lockKey);
        try {
            boolean isLock = rLock.tryLock(30, 10, TimeUnit.SECONDS);
            if (!isLock) {
                log.info("获取分布式锁失败，请重试");
                return false;
            }
            // 处理回调
            // 根据第三方id获取订单
            List<OrdersVO> ordersVOList = ordersService.getOrderListByThirdTradeNo(thirdTradeId);
            if (CollectionUtils.isEmpty(ordersVOList)) {
                log.error("根据第三方订单id查询不到订单信息，thirdTradeId：{}", thirdTradeId);
                // 根据自定义订单号查询
                ordersVOList = ordersService.getOrderByOrderNum(orderNum);
                if (CollectionUtils.isEmpty(ordersVOList)) {
                    log.error("根据自定义订单号查询不到订单信息，orderNum：{}", orderNum);
                    return false;
                }
            }
            // 获取用户订阅信息
            UserInfoVO userInfoVO = userApi.getMemberInfoById(ordersVOList.get(0).getUserId()).getResult();
            List<UserSubscriptionInfoVO> subscriptionInfoList = userInfoVO.getSubscriptionInfoList();
            // 获取产品信息
            ProductVO productVO = productApi.getProduct(ordersVOList.get(0).getProductId()).getResult();
            log.info("订单回调，产品信息，{}", productVO.toString());
            // 根据回调事件
            switch (eventType) {
                case "1":
                    // 买断支付成功，订阅首期支付成功
                    // 判断order，买断和订阅首期支付不会有多条order记录
                    if (ordersVOList.size() != 1) {
                        log.error("支付回调错误！查询到多条订单信息，thirdTradeId：{}，orderNum", thirdTradeId, orderNum);
                        return false;
                    }
                    OrdersVO ordersVO = ordersVOList.get(0);
                    log.info("支付成功回调，订单：{}", ordersVO);
                    if (ordersVO.getStatus().equals(OrderConstant.COMPLETED)) {
                        log.error("支付的订单已完成，id:{}", ordersVO.getId());
                        return false;
                    }
                    // 插入subscription
                    Subscription subscription = new Subscription();
                    subscription.setId(CommonUtils.generateId());
                    subscription.setProductId(ordersVO.getProductId());
                    subscription.setStatus(SubscriptionConstant.actived);
                    if(ObjectUtils.isNotEmpty(ordersVO.getReducedPrice())){
                        subscription.setPrice(ordersVO.getReducedPrice());
                    }else{
                        subscription.setPrice(ordersVO.getPrice());
                    }
                    subscription.setUserId(ordersVO.getUserId());
                    subscription.setEmail(ordersVO.getEmail());
                    subscription.setSubscriptionType(ordersVO.getSubscriptionType());
                    subscription.setAppId(ordersVO.getAppId());
                    subscription.setPaymentModel(ordersVO.getPaymentModel());
                    subscription.setThirdSubscriptionId(thirdTradeId);
                    subscription.setOrderId(ordersVO.getId());
                    subscription.setPayment(ordersVO.getPayment());
                    subscription.setBuyerType(2);
                    // 订阅订单，设置订阅开始时间和结束时间
                    if (ordersVO.getPaymentModel() == 1) {
                        // 获取当前时间
                        Date currentDate = new Date();
                        subscription.setStartDate(currentDate);
                        Date endDate ;
                        if(productVO.getCode().contains("annual-subscription-trail")){
                            Calendar calendar = Calendar.getInstance();
                            calendar.setTime(currentDate);
                            calendar.add(Calendar.DAY_OF_MONTH, 7);
                            endDate = calendar.getTime();
                        }else{
                            endDate = getDateByCycle(currentDate, productVO.getCycle());
                        }
                        subscription.setEndDate(endDate);
                        // 发送订阅过期消息
                        SubscriptionUtil.subscriptionExpiredSendNews(rabbitTemplate, subscription.getId(), endDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime());
                    }
                    subscriptionsService.save(subscription);
                    // 更新订单状态
                    Order order = new Order();
                    order.setId(ordersVO.getId());
                    order.setSubscriptionId(subscription.getId());
                    order.setThirdOrderNo(payId);
                    order.setThirdTradeNo(thirdTradeId);
                    order.setStatus(OrderConstant.COMPLETED);
                    order.setPayDate(new Date());
                    order.setInvoiceNo(MyDateUtils.getTimeStamp() + (int) ((Math.random() * 9 + 1) * 1000));
                    ordersService.updateById(order);
                    ordersVO.setInvoiceNo(order.getInvoiceNo());
                    // 更新user_subscription_info
                    UserSubscriptionInfo userSubscriptionInfo = new UserSubscriptionInfo();

                    boolean emailFlag = false;
                    // 如果是批量的购买，不需要判断用户有没有购买，直接存到订单赠送
                    if (productVO.getPlatform() == 1 && ordersVO.getPayNumber() > 1) {
                        log.info("批量购买{}个，新增订单赠送", ordersVO.getPayNumber());
                        List<OrderGift> orderGiftList = new ArrayList<>();
                        for (Integer i = 0; i < ordersVO.getPayNumber(); i++) {
                            OrderGift orderGift = new OrderGift();
                            orderGift.setId(CommonUtils.generateId());
                            orderGift.setOrderId(ordersVO.getId());
                            orderGift.setProductId(ordersVO.getProductId());
                            orderGift.setUserId(ordersVO.getUserId());
                            orderGift.setValidFlag("1");
                            orderGiftList.add(orderGift);
                        }
                        orderGiftService.saveBatch(orderGiftList);
                    } else {
                        // 查询用户产品类型的订阅信息
                        List<UserSubscriptionInfoVO> collect = subscriptionInfoList.stream().filter(
                                userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatform().equals(productVO.getPlatform())
                        ).collect(Collectors.toList());
                        if (CollectionUtils.isNotEmpty(collect)) {
                            log.info("用户有订阅信息");
                            if (productVO.getPlatform() == 1) {
                                log.info("永久版产品，新增订单赠送", collect);
                                OrderGift orderGift = new OrderGift();
                                orderGift.setId(CommonUtils.generateId());
                                orderGift.setOrderId(ordersVO.getId());
                                orderGift.setProductId(ordersVO.getProductId());
                                orderGift.setUserId(ordersVO.getUserId());
                                orderGift.setValidFlag("1");
                                orderGiftService.save(orderGift);
                            } else {
                                log.info("订阅版产品，更新用户订阅信息");
                                UserSubscriptionInfoVO userSubscriptionInfoVO = collect.get(0);
                                BeanUtil.copyProperties(userSubscriptionInfoVO, userSubscriptionInfo);
                                userSubscriptionInfo.setStatus(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value());
                                // 获取当前时间
                                Date currentDate = new Date();
                                // 根据产品周期添加订阅信息记录的结束时间
                                if (currentDate.compareTo(userSubscriptionInfoVO.getEndDate()) < 0) {
                                    // 订阅未过期，设置结束时间为当前时间加产品周期
                                    currentDate = userSubscriptionInfoVO.getEndDate();
                                }
                                Date endDate = getDateByCycle(currentDate, productVO.getCycle());
                                userSubscriptionInfo.setEndDate(endDate);
                                userSubscriptionInfo.setPayType(1);
                                userSubscriptionInfo.setPlatform(productVO.getPlatform());
                                userSubscriptionInfo.setAppId(ordersVO.getAppId());
                                userSubscriptionInfo.setProductId(ordersVO.getProductId());
                                UserVO userVO = new UserVO();
                                userVO.setId(ordersVO.getUserId());
                                if(productVO.getPlatform()==3){
                                    //如果是ai还要加ai点数
                                    userSubscriptionInfo.setPoint(userSubscriptionInfo.getPoint()+Integer.valueOf(productVO.getPoints()));
                                    if(productVO.getCode().contains("trail")){
                                        userVO.setCanAccessAi("0");
                                    }
                                } else {
                                    userVO.setAccountType(AccountTypeEnum.PAID_ACCOUNT.value());
                                }
                                userApi.updateUser(userVO);
                                userSubscriptionInfoApi.update(userSubscriptionInfo);
                                emailFlag = true;
                            }
                        } else {
                            log.info("用户没有订阅信息，新增记录");
                            userSubscriptionInfo.setId(CommonUtils.generateId());
                            userSubscriptionInfo.setUserId(ordersVO.getUserId());
                            userSubscriptionInfo.setStatus(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value());
                            userSubscriptionInfo.setAppId(ordersVO.getAppId());
                            userSubscriptionInfo.setProductId(ordersVO.getProductId());
                            //0免费，1永久版，2订阅，3ai
                            userSubscriptionInfo.setPlatform(productVO.getPlatform());
                            if (productVO.getPlatform() == 1) {
                                // 设置过期时间一百年
                                // 获取当前时间
                                Calendar calendar = Calendar.getInstance();
                                // 向当前时间增加100年
                                calendar.add(Calendar.YEAR, 100);
                                userSubscriptionInfo.setEndDate(calendar.getTime());
                                userSubscriptionInfo.setPayType(0);
                            } else {
                                // 获取当前时间
                                Date currentDate = new Date();
                                Date endDate ;
                                if(productVO.getCode().contains("annual-subscription-trail")){
                                    Calendar calendar = Calendar.getInstance();
                                    calendar.setTime(currentDate);
                                    calendar.add(Calendar.DAY_OF_MONTH, 7);
                                    endDate = calendar.getTime();
                                }else{
                                    endDate = getDateByCycle(currentDate, productVO.getCycle());
                                }
                                userSubscriptionInfo.setEndDate(endDate);
                                userSubscriptionInfo.setPayType(1);
                            }
                            UserVO userVO = new UserVO();
                            userVO.setId(ordersVO.getUserId());
                            if(productVO.getPlatform()==3){
                                //如果是ai还要加ai点数
                                userSubscriptionInfo.setPoint(Integer.valueOf(productVO.getPoints()));
                                if(productVO.getCode().contains("trail")){
                                    userVO.setCanAccessAi("0");
                                    userApi.updateUser(userVO);
                                }
                            } else {
                                userVO.setAccountType(AccountTypeEnum.PAID_ACCOUNT.value());
                                userApi.updateUser(userVO);
                            }
                            userSubscriptionInfoApi.insert(userSubscriptionInfo);
                            emailFlag = true;
                        }
                    }
                    //删除缓存中的订单
                    ordersService.deleteTheCacheInTheCacheManager(ordersVO.getUserId() + "-" + ordersVO.getProductId() + "-" + ordersVO.getPayment() + "-" + ordersVO.getPayNumber() + "-" + ordersVO.getDiscountType());
                    // 关闭用户同类型订单
                    ordersService.closeOrderByUser(ordersVO.getUserId(), productVO.getPlatform());
                    if (emailFlag) {
                        log.info("用户订阅或支付成功，发送订阅邮件");
                        EmailSendBO bo = new EmailSendBO();
                        bo.setToEmail(userInfoVO.getEmail());
                        //设置事件
                        if (ordersVO.getDiscountType() == 2) {
                            bo.setUseEvent(EmailCodeTypeEnum.MEMBER_UPGRADE_SUCCESS.value());
                        } else {
                            bo.setUseEvent(EmailCodeTypeEnum.MEMBER_BUY_SUCCESS.value());
                        }
                        //设置title
                        Map<String,String> titleMap = new HashMap<>();
                        bo.setSendTitleContent(titleMap);
                        //设置内容
                        Map<String,String> contentMap = new HashMap<>();
                        contentMap.put("@plan@", productVO.getProductName());
                        Date date = new Date();
                        // 创建 SimpleDateFormat 对象，指定日期格式
                        SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
                        // 格式化当前日期
                        String formattedDate = sdf.format(date);
                        contentMap.put("@date@",formattedDate);
                        String price = ObjectUtils.isNotEmpty(ordersVO.getReducedPrice())? ordersVO.getReducedPrice().toEngineeringString() : ordersVO.getPrice().toEngineeringString();
                        String currencySign;
                        if (ordersVO.getPayment() == 1 || ordersVO.getPayment() == 2) {
                            currencySign = "CNY";
                        } else {
                            currencySign = "USD";
                        }
                        contentMap.put("@payPrice@", currencySign + price);
                        contentMap.put("@renewPrice@", currencySign + price);
                        bo.setSendContent(contentMap);
                        log.info("获取发票");
                        String invoiceHtml = null;
                        try {
                            invoiceHtml = new String(StreamUtils.copyToByteArray(this.getClass().getClassLoader().getResourceAsStream("templates/invoice_member.html")), StandardCharsets.UTF_8);
                        } catch (IOException e) {
                            log.error(e.getMessage());
                        }
                        Map<String, Object> map = new HashMap<>();
                        map.put("Name", userInfoVO.getEmail());
                        map.put("email", userInfoVO.getEmail());
                        map.put("purchasedDate", formattedDate);
                        map.put("orderNumber", ordersVO.getTradeNo());
                        map.put("invoiceDate", formattedDate);
                        map.put("invoiceNumber", ordersVO.getInvoiceNo());
                        map.put("quantity", ordersVO.getPayNumber());
                        map.put("unitPrice", productVO.getPrice().toEngineeringString());
                        String discount = "0";
                        String amount = ordersVO.getPrice().toEngineeringString();
                        if (ordersVO.getReducedPrice() != null) {
                            /*if (ordersVO.getPayNumber() > 0) {
                                BigDecimal totalPrice = productVO.getPrice().multiply(BigDecimal.valueOf(ordersVO.getPayNumber()));
                                discount = totalPrice.subtract(ordersVO.getReducedPrice()).toEngineeringString();
                            } else {
                                discount = productVO.getPrice().subtract(ordersVO.getReducedPrice()).toEngineeringString();
                            }*/
                            discount = productVO.getPrice().subtract(ordersVO.getReducedPrice()).toEngineeringString();
                            amount = ordersVO.getReducedPrice().toEngineeringString();
                        }
                        map.put("discount", discount);
                        map.put("amount", amount);
                        map.put("total", amount);
                        log.info("发票html内容填充map:{}", map);
                        // 生成替换内容后的发票
                        String htmlStr = TemplatesUtil.replaceStringUsingFreeMarker(invoiceHtml, map);
                        // 微信支付宝币种改为 CNY
                        if (ordersVO.getPayment() == 1 || ordersVO.getPayment() == 2) {
                            htmlStr = htmlStr.replace("USD", "CNY");
                        }
                        HttpHeaders headers = new HttpHeaders();
                        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
                        // 封装表单数据
                        MultiValueMap<String, String> paramMap = new LinkedMultiValueMap<>();
                        paramMap.add("html", htmlStr);
                        HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(paramMap, headers);
                        // 发送POST请求,html转pdf
                        Map postForObjectMap = restTemplate.postForObject(htmlToPdfUrl, request, Map.class);
                        String invoiceFileUrl = postForObjectMap.get("url").toString();
                        log.info("发票文件地址：{}", invoiceFileUrl);
                        bo.setFileUrl(invoiceFileUrl);
                        bo.setFileName("invoice.pdf");
                        log.info("发送订阅邮件");
                        emailApi.sendEmail(bo);
                    }
                    break;
                case "2":
                    // 支付失败，有延时队列暂不处理
                    log.info("支付失败，有延时队列暂不处理，thirdTradeId：{}", thirdTradeId);
                    break;
                case "3":
                    // 订阅周期扣款成功
                    // 新增order
                    Order cycleOrder = new Order();
                    BeanUtil.copyProperties(ordersVOList.get(0), cycleOrder);
                    cycleOrder.setId(CommonUtils.generateId());
                    cycleOrder.setTradeNo(cn.kdan.cloud.pdf.office.payment.utils.CommonUtils.generateRightsId((long) ((Math.random() * 9999 + 1) * 10000)));
                    cycleOrder.setThirdOrderNo(payId);
                    ordersService.save(cycleOrder);
                    // 通过thirdTradeId更新subscription
                    Subscription serviceOne = subscriptionsService.getOne(new LambdaQueryWrapper<Subscription>().eq(Subscription::getThirdSubscriptionId, thirdTradeId));
                    serviceOne.setPrice(cycleOrder.getPrice());
                    serviceOne.setStartDate(new Date());
                    serviceOne.setEndDate(new Date(nextBillingTime));
                    subscriptionsService.updateById(serviceOne);
                    // 更新user_subscription_info
                    UserSubscriptionInfo userSubscriptionInfo1 = new UserSubscriptionInfo();
                    // 查询用户订阅的信息
                    List<UserSubscriptionInfoVO> collect = subscriptionInfoList.stream().filter(userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatform().equals(productVO.getPlatform())).collect(Collectors.toList());
                    if (CollectionUtils.isNotEmpty(collect)) {
                        log.error("订阅周期付款，用户订阅信息异常，thirdTradeId：{}", thirdTradeId);
                        break;
                    }
                    BeanUtil.copyProperties(collect.get(0), userSubscriptionInfo1);
                    // 获取当前时间
                    Date currentDate = new Date();
                    // 根据产品周期添加订阅信息记录的结束时间
                    if (currentDate.compareTo(userSubscriptionInfo1.getEndDate()) < 0) {
                        // 订阅未过期，设置结束时间为当前时间加产品周期
                        currentDate = userSubscriptionInfo1.getEndDate();
                    }
                    Date endDate = getDateByCycle(currentDate, productVO.getCycle());
                    userSubscriptionInfo1.setEndDate(endDate);
                    userSubscriptionInfoApi.update(userSubscriptionInfo1);
                    // 订阅过期信息
                    // 发送订阅过期消息
                    SubscriptionUtil.subscriptionExpiredSendNews(rabbitTemplate, userSubscriptionInfo1.getId(), endDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime());
                case "4":
                    // 订阅扣款失败，有延时队列暂不处理
                    log.info("订阅扣款失败，有延时队列暂不处理，thirdTradeId：{}", thirdTradeId);
                    break;
                case "5":
                    // 取消订阅，更新subscription
                    log.info("用户取消订阅，更新subscription，thirdTradeId：{}", thirdTradeId);
                    // 通过thirdTradeId更新subscription
                    Subscription serviceOne1 = subscriptionsService.getOne(new LambdaQueryWrapper<Subscription>().eq(Subscription::getThirdSubscriptionId, thirdTradeId));
                    serviceOne1.setSubscriptionType(0);
                    subscriptionsService.updateById(serviceOne1);
                    // 更新user_subscription_info
                    UserSubscriptionInfo userSubscriptionInfo2 = new UserSubscriptionInfo();
                    // 查询用户订阅的信息
                    List<UserSubscriptionInfoVO> collect1 = subscriptionInfoList.stream().filter(userSubscriptionInfoVO -> userSubscriptionInfoVO.getPlatform().equals(productVO.getPlatform())).collect(Collectors.toList());
                    if (CollectionUtils.isEmpty(collect1)) {
                        log.error("订阅取消，用户订阅信息异常，thirdTradeId：{}", thirdTradeId);
                        break;
                    }
                    log.info("用户取消订阅信息：{}", collect1.get(0));
                    BeanUtil.copyProperties(collect1.get(0), userSubscriptionInfo2);
                    userSubscriptionInfo2.setPayType(PDFOfficeUserSubscriptionPayTypeEnum.NON_AUTO_SUBSCRIPTION.value());
                    userSubscriptionInfoApi.update(userSubscriptionInfo2);
                    break;
            }
        } catch (Exception e) {
            log.error("获取分布式锁异常，{}", e.getStackTrace());
            return false;
        } finally {
            if (rLock.isLocked() && rLock.isHeldByCurrentThread()) {
                rLock.unlock();
            }
        }
        return true;
    }

    @NotNull
    private static Date getDateByCycle(Date startDate, Integer cycle) {
        // 创建 Calendar 实例并设置为当前时间
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startDate);
        // 根据产品周期设置结束时间
        switch (cycle) {
            case 1:
                calendar.add(Calendar.MONTH, 1);
                break;
            case 2:
                calendar.add(Calendar.MONTH, 3);
                break;
            case 3:
                calendar.add(Calendar.MONTH, 6);
                break;
            case 4:
                calendar.add(Calendar.YEAR, 1);
                break;
        }
        return calendar.getTime();
    }

}
