package cn.kdan.cloud.pdf.office.website.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.extra.qrcode.QrCodeUtil;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.feign.OrderApi;
import cn.kdan.cloud.pdf.office.api.payment.feign.SubscriptionApi;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ListingProductVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.dto.CreateUserOrderDTO;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserStatusEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.website.service.OrderService;
import cn.kdan.cloud.pdf.office.website.service.ProductInfoService;
import cn.kdan.cloud.pdf.office.website.utils.SecurityUtils;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.math.BigDecimal;
import java.util.Base64;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-WPH 2023/2/9
 */
@Service
@Slf4j
@RequiredArgsConstructor
@Transactional
public class OrderServiceImpl implements OrderService {

    private final SubscriptionApi subscriptionApi;

    private final OrderApi orderApi;

    private final UserApi userApi;

    private final ProductApi productApi;

    private final UserSubscriptionInfoApi userSubscriptionInfoApi;

    private final ProductInfoService productInfoService;

    @Override
    public Map<String, String> createOrder(CreateUserOrderDTO createOrderDTO) {
        CustomUserDetails user = SecurityUtils.getUser();
        createOrderDTO.setUserId(user.getId());
        // 判断用户状态
        UserVO userVO = userApi.getById(user.getId()).getResult();
        if (userVO.getValidFlag().equals(PDFOfficeUserStatusEnum.LOGGED_OUT_ING.value().toString())) {
            log.error("用户注销中，无法购买，{}", user.getId());
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_ABNORMAL_USER_STATUS);
        }
        // 优惠券校验
        boolean couponFlag = false;
        // 批量购买时，校验价格
        if (createOrderDTO.getNum() > 1) {
            if (createOrderDTO.getDiscountFlag() == 4) {
                // 查询商品及批量价格
                Map<String, String> batchProductPrice = productInfoService.getBatchProductPrice(createOrderDTO.getProductId(), createOrderDTO.getNum());
                // 判断币种
                BigDecimal totalPrice;
                if (createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.PAYPAL.getValue()) || createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.PADDLE.getValue())) {
                    totalPrice = new BigDecimal(batchProductPrice.get("totalPrice"));
                } else {
                    totalPrice = new BigDecimal(batchProductPrice.get("cnyTotalPrice"));
                }
                if (totalPrice.compareTo(createOrderDTO.getPrice()) != 0) {
                    log.error("批量购买价格校验失败，商品id：{}，购买数量：{}，批量计算价格：{}，用户传入价格：{}", createOrderDTO.getProductId(), createOrderDTO.getNum(), batchProductPrice, createOrderDTO.getPrice());
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
                }
            } else if (createOrderDTO.getDiscountFlag() == 5) {
                // 获取商品
                ProductVO productVO = productApi.getProduct(createOrderDTO.getProductId()).getResult();
                // 判断币种
                BigDecimal totalPrice;
                if (createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.PAYPAL.getValue()) || createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.PADDLE.getValue())) {
                    totalPrice = productVO.getDisplayPrice().multiply(BigDecimal.valueOf(createOrderDTO.getNum()));
                } else {
                    totalPrice = productVO.getCnyDisplayPrice().multiply(BigDecimal.valueOf(createOrderDTO.getNum()));
                }
                if (totalPrice.compareTo(createOrderDTO.getPrice())  != 0) {
                    log.error("黑五批量购买价格校验失败，商品id：{}，购买数量：{}，批量计算价格：{}，用户传入价格：{}", createOrderDTO.getProductId(), createOrderDTO.getNum(), totalPrice, createOrderDTO.getPrice());
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
                }
            } else {
                log.error("购买数量错误，{}", createOrderDTO);
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
            }

        } else {
            // 单个购买，校验价格
            // 教育优惠和原价，判断商品价格
            if (createOrderDTO.getDiscountFlag() == 3 || createOrderDTO.getDiscountFlag() == 0) {
                int isEducation = createOrderDTO.getDiscountFlag() == 3 ? 1 : 0;
                ListingProductVO productPriceForBuy = productInfoService.getProductPriceForBuy(createOrderDTO.getProductId(), createOrderDTO.getUserId(), isEducation);
                // 根据支付方式验证价格
                if (createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.PAYPAL.getValue()) || createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.PADDLE.getValue())) {
                    if (productPriceForBuy.getPrice() == null || productPriceForBuy.getPrice().compareTo(createOrderDTO.getPrice()) != 0) {
                        log.error("购买美元价格校验失败，商品id：{}，用户id：{}，商品价格：{}，用户购买价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), productPriceForBuy.getPrice(), createOrderDTO.getPrice());
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_IS_NOT_EDUCATION_USER);
                    }
                } else {
                    if (productPriceForBuy.getCnyPrice() == null || productPriceForBuy.getCnyPrice().compareTo(createOrderDTO.getPrice()) != 0) {
                        log.error("购买人民币价格校验失败，商品id：{}，用户id：{}，商品价格：{}，用户购买价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), productPriceForBuy.getCnyPrice(), createOrderDTO.getPrice());
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_USER_IS_NOT_EDUCATION_USER);
                    }
                }
            } else if (createOrderDTO.getDiscountFlag() == 2) {
                // 买断暂无升级升级购买
                log.error("买断接口暂无升级，{}", createOrderDTO);
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_CREATE_ORDER_PARAMETER);
//            ListingProductVO productPriceForBuy = productInfoService.getProductPriceForBuy(createOrderDTO.getProductId(), createOrderDTO.getUserId(), 0);
//            if (productPriceForBuy.getUpgradePrice() == null || productPriceForBuy.getUpgradePrice().compareTo(createOrderDTO.getPrice()) != 0) {
//                log.error("升级购买价格校验失败，商品id：{}，用户id：{}，升级价格：{}，用户传入价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), productPriceForBuy.getUpgradePrice(), createOrderDTO.getPrice());
//                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
//            }
            } else if (createOrderDTO.getDiscountFlag() == 1 || createOrderDTO.getDiscountFlag() == 11) {
                if (createOrderDTO.getCouponCode() == null) {
                    log.error("优惠券校验失败，优惠券为空，{}", createOrderDTO);
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_NOT_EXISTS);
                }
                ListingProductVO checkCoupon = productInfoService.checkCoupon(createOrderDTO.getProductId(), createOrderDTO.getUserId(), createOrderDTO.getCouponCode()).getResult();
                if (null == checkCoupon) {
                    log.error("优惠券校验失败，code:{}", createOrderDTO.getCouponCode());
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_NOT_EXISTS);
                }
                // 根据支付方式验证价格
                if (createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.PAYPAL.getValue()) || createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.PADDLE.getValue())) {
                    if (checkCoupon.getDisplayPrice() == null || checkCoupon.getDisplayPrice().compareTo(createOrderDTO.getPrice()) != 0) {
                        log.error("购买时使用优惠券校验美元价格失败，商品id：{}，用户id：{}，用户传入价格：{}, 优惠券信息价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), createOrderDTO.getPrice(), checkCoupon);
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_PRODUCT_ERROR);
                    }
                } else {
                    if (checkCoupon.getCnyDisplayPrice() == null || checkCoupon.getCnyDisplayPrice().compareTo(createOrderDTO.getPrice()) != 0) {
                        log.error("购买时使用优惠券校验人民币价格失败，商品id：{}，用户id：{}，用户传入价格：{}, 优惠券信息价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), createOrderDTO.getPrice(), checkCoupon);
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_PRODUCT_ERROR);
                    }
                }
                // 验证优惠券成功，标记更新优惠券状态
                couponFlag = true;
            }
        }

        // 价格校验通过，调用order创建
        Map<String, String> result = orderApi.createUserOrder(createOrderDTO).getResult();
        String payHref = result.get("payHref");
        if (StringUtils.isEmpty(payHref)) {
            log.error("创建订单失败，{}", createOrderDTO);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_ORDER_CREATE_FAILED);
        }
        if (couponFlag) {
            // 更新优惠券状态
            productInfoService.updateCouponState(createOrderDTO.getCouponCode(), 2, result.get("orderId"));
        }
        log.info("创建订单成功，{}", result);
        // 微信支付宝链接生成二维码
        if (createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.ALIPAY.getValue()) || createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.WXPAY.getValue())) {
            String qrcode = generateQrcode(payHref);
            result.put("qrcode", qrcode);
        }

        return result;
    }

    private static String generateQrcode(String payHref) {
        log.info("生成二维码，保存为临时文件：{}", payHref);
        // 生成二维码并保存为临时文件
        String qrcodeFileUrl = System.getProperty("java.io.tmpdir") + "/qrcode-" + UUID.randomUUID().toString().replace("-", "") + ".png";
        log.info("二维码文件路径：{}", qrcodeFileUrl);
        File qrCodeFile = new File(qrcodeFileUrl);
        QrCodeUtil.generate(payHref, 300, 300, qrCodeFile);
        try {
            log.info("将二维码图片转换为 Base64 编码");
            // 读取二维码图片
            BufferedImage bufferedImage = ImageIO.read(qrCodeFile);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // 将图片写入输出流
            ImageIO.write(bufferedImage, "png", baos);
            baos.flush();
            byte[] bytes = baos.toByteArray();
            String base64 = Base64.getEncoder().encodeToString(bytes);
            return "data:image/png;base64," + base64;
        } catch (Exception e) {
            log.error("生成二维码失败，{}", e.getMessage());
            return null;
        } finally {
            // 删除临时文件
            log.info("删除二维码临时文件");
            if (qrCodeFile.exists()) {
                if (qrCodeFile.delete()) {
                    log.info("删除二维码临时文件成功");
                }
            }
        }
    }

    @Override
    public Map<String, String> createSubscription(CreateUserOrderDTO createOrderDTO) {
        // 订阅目前支持 PayPal paddle
        if (createOrderDTO.getPaymentMethod() != 0 && createOrderDTO.getPaymentMethod() != 3) {
            log.error("订阅目前只支持 paypal 和 paddle，{}", createOrderDTO);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_CREATE_ORDER_PARAMETER);
        }
        // 不支持批量购买
        if (createOrderDTO.getDiscountFlag() == 4 || createOrderDTO.getNum() != 1) {
            log.error("订阅目前不支持批量购买，{}", createOrderDTO);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
        }
        CustomUserDetails user = SecurityUtils.getUser();
        createOrderDTO.setUserId(user.getId());
        // 判断用户状态
        UserVO userVO = userApi.getById(user.getId()).getResult();
        if (userVO.getValidFlag().equals(PDFOfficeUserStatusEnum.LOGGED_OUT_ING.value().toString())) {
            log.error("用户注销中，无法购买，{}", user.getId());
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_ABNORMAL_USER_STATUS);
        }
        // 教育优惠升级购买
        if (createOrderDTO.getDiscountFlag() == 10) {
            ListingProductVO productPriceForBuy = productInfoService.getProductPriceForBuy(createOrderDTO.getProductId(), createOrderDTO.getUserId(), 1);
            if (productPriceForBuy.getUpgradePrice() == null || productPriceForBuy.getUpgradePrice().compareTo(createOrderDTO.getPrice()) != 0) {
                log.error("教育优惠升级购买价格校验失败，商品id：{}，用户id：{}，教育升级价格：{}，用户传入价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), productPriceForBuy.getUpgradePrice(), createOrderDTO.getPrice());
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
            }
        }
        // 普通升级购买
        if (createOrderDTO.getDiscountFlag() == 2) {
            ListingProductVO productPriceForBuy = productInfoService.getProductPriceForBuy(createOrderDTO.getProductId(), createOrderDTO.getUserId(), 0);
            if (productPriceForBuy.getUpgradePrice() == null || productPriceForBuy.getUpgradePrice().compareTo(createOrderDTO.getPrice()) != 0) {
                log.error("升级购买价格校验失败，商品id：{}，用户id：{}，升级价格：{}，用户传入价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), productPriceForBuy.getUpgradePrice(), createOrderDTO.getPrice());
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
            }
        }
        // 优惠券
        boolean couponFlag = false;
        if (createOrderDTO.getDiscountFlag() == 1 || createOrderDTO.getDiscountFlag() == 11) {
            if (createOrderDTO.getCouponCode() == null) {
                log.error("优惠券校验失败，优惠券为空，{}", createOrderDTO);
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_NOT_EXISTS);
            }
            ListingProductVO checkCoupon = productInfoService.checkCoupon(createOrderDTO.getProductId(), createOrderDTO.getUserId(), createOrderDTO.getCouponCode()).getResult();
            if (null == checkCoupon) {
                log.error("优惠券校验失败，code:{}", createOrderDTO.getCouponCode());
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_NOT_EXISTS);
            }
            // 订阅只校验美元价格
            if (checkCoupon.getDisplayPrice() == null || checkCoupon.getDisplayPrice().compareTo(createOrderDTO.getPrice()) != 0) {
                log.error("购买订阅使用优惠券校验美元价格失败，商品id：{}，用户id：{}，用户购买价格：{}, 优惠券信息价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), createOrderDTO.getPrice(), checkCoupon);
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_PRODUCT_ERROR);
            }
            // 验证优惠券成功，标记更新优惠券状态
            couponFlag = true;
        }
        // 教育优惠和原价，判断商品价格
        if (createOrderDTO.getDiscountFlag() == 3 || createOrderDTO.getDiscountFlag() == 0) {
            int isEducation = createOrderDTO.getDiscountFlag() == 3 ? 1 : 0;
            ListingProductVO productPriceForBuy = productInfoService.getProductPriceForBuy(createOrderDTO.getProductId(), createOrderDTO.getUserId(), isEducation);
            BigDecimal price = !ObjectUtils.isEmpty(productPriceForBuy.getDisplayPrice())?productPriceForBuy.getDisplayPrice():productPriceForBuy.getPrice();
            if (productPriceForBuy.getPrice() == null || productPriceForBuy.getPrice().compareTo(price) != 0) {
                log.error("购买订阅价格校验失败，商品id：{}，用户id：{}，商品价格：{}，用户购买价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), productPriceForBuy.getPrice(), createOrderDTO.getPrice());
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
            }
        }
        //首期优惠包括黑五和0元领取ai
        if (createOrderDTO.getDiscountFlag() == 5) {
            if(StringUtils.isNotEmpty(createOrderDTO.getCouponCode())){
                ListingProductVO checkCoupon = productInfoService.checkCoupon(createOrderDTO.getProductId(), createOrderDTO.getUserId(), createOrderDTO.getCouponCode()).getResult();
                if (null == checkCoupon) {
                    log.error("优惠券校验失败，code:{}", createOrderDTO.getCouponCode());
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_NOT_EXISTS);
                }
                // 订阅只校验美元价格
                if (checkCoupon.getDisplayPrice() == null || checkCoupon.getDisplayPrice().compareTo(createOrderDTO.getPrice()) != 0) {
                    log.error("购买订阅使用优惠券校验美元价格失败，商品id：{}，用户id：{}，用户购买价格：{}, 优惠券信息价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), createOrderDTO.getPrice(), checkCoupon);
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_PRODUCT_ERROR);
                }
                // 验证优惠券成功，标记更新优惠券状态
                couponFlag = true;
            }else{
                ListingProductVO productPriceForBuy = productInfoService.getProductPriceForBuy(createOrderDTO.getProductId(), createOrderDTO.getUserId(), 0);
                if(productPriceForBuy.getCode().equals("ai-subscription-year-trail")&&!subscriptionApi.judgeAiDiscount(user.getId()).getResult()){
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_ORDER_CREATE_FAILED);
                }
                if (productPriceForBuy.getPrice() == null) {
                    log.error("购买订阅价格校验失败，商品id：{}，用户id：{}，商品价格：{}，用户购买价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), productPriceForBuy.getPrice(), createOrderDTO.getPrice());
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
                }
                //如果有优惠价，就用优惠价
                BigDecimal price = !ObjectUtils.isEmpty(productPriceForBuy.getDisplayPrice())?productPriceForBuy.getDisplayPrice():productPriceForBuy.getPrice();
                if (price.compareTo(createOrderDTO.getPrice()) != 0) {
                    log.error("购买订阅价格校验失败，商品id：{}，用户id：{}，商品价格：{}，用户购买价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), productPriceForBuy.getPrice(), createOrderDTO.getPrice());
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
                }
            }
        }
        // 价格校验通过，调用order创建
        Map<String, String> result = orderApi.createUserSubscription(createOrderDTO).getResult();
        if (StringUtils.isEmpty(result.get("payHref"))) {
            log.error("创建订阅订单失败，{}", createOrderDTO);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_ORDER_CREATE_FAILED);
        }
        if (couponFlag) {
            // 更新优惠券状态
            productInfoService.updateCouponState(createOrderDTO.getCouponCode(), 2, result.get("orderId"));
        }
        return result;
    }

    @Override
    public Map<String, String> getStateByOrderId(String orderId) {
        return orderApi.getStateByOrderId(orderId).getResult();
    }

    @Override
    public List<OrdersVO> getOrderListByStatus(Integer orderStatus) {
        List<OrdersVO> ordersVOList = orderApi.getOrderListByStatus(orderStatus, SecurityUtils.getUserId()).getResult();
        List<String> productIds = ordersVOList.stream().map(OrdersVO::getProductId).distinct().collect(Collectors.toList());
        List<ListingProductVO> productVOList = productApi.getProductsByIds(productIds).getResult();
        for (OrdersVO ordersVO : ordersVOList) {
            List<ListingProductVO> collect = productVOList.stream().filter(productVO -> productVO.getId().equals(ordersVO.getProductId())).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(collect)) {
                continue;
            }
            ListingProductVO listingProductVO = collect.get(0);
            ordersVO.setProductName(listingProductVO.getProductName());
            ordersVO.setProductCycle(listingProductVO.getCycle());
            ordersVO.setProductIsAi(listingProductVO.getIsAi());
            ordersVO.setSimplifiedChineseName(listingProductVO.getSimplifiedChineseName());
            ordersVO.setTraditionalChineseName(listingProductVO.getTraditionalChineseName());
            if (ordersVO.getStatus() == 0 && (ordersVO.getPayment() == 1 || ordersVO.getPayment() == 2) ) {
                String pagePayUrl = ordersVO.getPagePayUrl();
                String qrcode = generateQrcode(pagePayUrl);
                ordersVO.setQrcode(qrcode);
            }
        }
        return ordersVOList;
    }

    @Override
    public Boolean closeOrder(String orderId) {
        return orderApi.closeOrder(orderId).getResult();
    }
}
