package cn.kdan.cloud.pdf.office.sso.controller;

import cn.kdan.cloud.pdf.office.api.account.vo.LoginDevice;
import cn.kdan.cloud.pdf.office.api.email.feign.EmailApi;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.dto.ResetPasswordDTO;
import cn.kdan.cloud.pdf.office.common.dto.UserRegisterDTO;
import cn.kdan.cloud.pdf.office.common.dto.UserResetPasswordDTO;
import cn.kdan.cloud.pdf.office.common.dto.ValidUserDTO;
import cn.kdan.cloud.pdf.office.common.enums.EmailCodeTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.vo.TokenVO;
import cn.kdan.cloud.pdf.office.sso.enums.VerifyTypeEnum;
import cn.kdan.cloud.pdf.office.sso.pojo.LoginDeviceMapVO;
import cn.kdan.cloud.pdf.office.sso.pojo.LoginParam;
import cn.kdan.cloud.pdf.office.sso.service.AuthService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.web.bind.annotation.*;

import javax.crypto.Mac;
import javax.servlet.http.HttpServletRequest;
import javax.validation.constraints.NotBlank;
import java.security.Principal;
import java.util.List;
import java.util.Locale;

/**
 * 登录模块
 */
@RestController
@RequestMapping("/auth")
public class AuthController {

    @Autowired
    private EmailApi emailApi;

    private final AuthService authService;

    @Autowired
    public AuthController(AuthService authService) {
        this.authService = authService;
    }


    /**
     * @param clientId
     * @param principal
     * @param request
     * @Description: 获取授权码，登陆时调用
     * @Return: com.wondersgroup.cloud.cmp.common.pojo.ResultMap<java.lang.String>
     * @Date: 2021/2/10 10:31
     */
    @GetMapping("/getAuthorizeCode")
    public ResultMap<String> authorize(@RequestParam String clientId, Principal principal, HttpServletRequest request) {
        String code = authService.getAuthorizeCode(getUserId(principal), clientId, request);
        if (StringUtils.isNotEmpty(code)) {
            return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, code);
        } else {
            return new ResultMap<>(ExceptionEnum.EXCEPTION_MSG_GET_CODE_FAIL.value(), ExceptionEnum.EXCEPTION_MSG_GET_CODE_FAIL.msg());
        }
    }

    /**
     * @param clientId clientId
     * @param code code
     * @param request request
     * @Description: 获取token  登陆时调用
     * @Date: 2021/2/10 10:32
     */
    @GetMapping("/getToken")
    public ResultMap<TokenVO> getToken(@RequestParam String clientId, @RequestParam String code, HttpServletRequest request) {
        TokenVO tokenVO = authService.getToken(code, clientId, request);
        if (tokenVO == null) {
            return new ResultMap<>(ExceptionEnum.EXCEPTION_MSG_GET_TOKEN_FAIL.value(), ExceptionEnum.EXCEPTION_MSG_GET_TOKEN_FAIL.msg());
        } else {
            return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, tokenVO);
        }
    }


    /**
     * 邮箱验证码登录
     * @param email 邮箱
     * @param password 验证码1
     * @param appId appid
     * @param platformType 平台类型 1 官网 2 后台
     * @param request
     * @return
     */
    @PostMapping("/emailLogin")
    public ResultMap<TokenVO> emailLogin(@RequestParam String email, @RequestParam String password, @RequestParam String appId,
                                       @RequestParam String platformType,  @RequestParam String deviceSign, @RequestParam(required = false) String model,HttpServletRequest request) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, authService.emailLogin(email,password,appId,platformType,deviceSign,model,request));
    }

    /**
     * 检查账号是否在其他设备登录，设备是否被自动登出
     * @param deviceSign  设备编号
     * @param appId 应用id
     */
    @PostMapping("/checkDeviceStatus")
    public ResultMap<Boolean> checkDeviceStatus( @RequestParam String appId,@RequestParam String deviceSign) {
        authService.checkDeviceStatus(deviceSign,appId);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }


    /**
     * 邮箱注册
     *
     * @param userRegisterDTO dto
     * @return TokenVO
     */
    @PostMapping("/emailRegister")
    public ResultMap<TokenVO> emailRegister(@RequestBody UserRegisterDTO userRegisterDTO) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, authService.emailRegister(userRegisterDTO));
    }

    @PostMapping("/resetPassword")
    public ResultMap<Boolean> resetPassword(@RequestBody UserResetPasswordDTO userResetPasswordDTO) {
        authService.resetPassword(userResetPasswordDTO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    /**
     * 新会员系统重置密码
     *
     * @param userResetPasswordDTO userResetPasswordDTO
     * @return
     */
    @PostMapping("/memberResetPassword")
    public ResultMap<Boolean> memberResetPassword(@RequestBody ResetPasswordDTO userResetPasswordDTO) {
        authService.memberResetPassword(userResetPasswordDTO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    /**
     * 验证用户是否存在，邮箱格式是否正确
     *
     * @param validUserDTO validUserDTO
     * @return
     */
    @PostMapping("/validUser")
    public ResultMap<Boolean> validUser(@RequestBody ValidUserDTO validUserDTO) {
        if ("1".equals(validUserDTO.getValidType())) {
            authService.validUserLoginParam(validUserDTO);
        } else {
            authService.validUserRegisterParam(validUserDTO);
        }
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

    /**
     * 验证邮箱（重置密码输入邮箱点击下一步时校验邮箱是否注册）
     *
     * @param email validEmail
     * @return
     */
    @PostMapping("/validEmail")
    public ResultMap<Boolean> validEmail(@RequestParam String email) {
        authService.validEmail(email);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }


    /**
     * 获取邮箱验证码（过期时间5分钟，重发间隔1分钟）
     *
     * @param action   验证码用途（login user_reset_password user_log_off）
     * @param type     0 email
     * @param receiver 邮箱号
     * @param appId    产品id
     * @return Boolean
     */
    @GetMapping("/getVerifyCode")
    public ResultMap<Boolean> getVerifyCode(@RequestParam String action,@RequestParam Integer type,@NotBlank(message = "receiver cannot be empty") @RequestParam String receiver,@RequestParam String appId) {
        EmailCodeTypeEnum actionEnum = EmailCodeTypeEnum.getEnumByValue(action);
        if (actionEnum == null) {
            return new ResultMap<>(ExceptionEnum.EXCEPTION_MSG_CODE_ACTION_ERROR.value(), ExceptionEnum.EXCEPTION_MSG_CODE_ACTION_ERROR.msg(), false);
        }
        if (StringUtils.isEmpty(receiver)) {
            return new ResultMap<>(ExceptionEnum.EXCEPTION_MSG_EMAIL_INVALID.value(), ExceptionEnum.EXCEPTION_MSG_EMAIL_INVALID.msg(), false);
        }
        VerifyTypeEnum verifyTypeEnum = VerifyTypeEnum.getEnumByCode(type);
        if (verifyTypeEnum == null) {
            return new ResultMap<>(CommonConstant.ERROR, CommonConstant.RESULT_ERROR, false);
        }
        if (authService.getVerifyCode(actionEnum, verifyTypeEnum, receiver,appId)) {
            return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, true);
        } else {
            return new ResultMap<>(VerifyTypeEnum.PHONE.code().equals(type) ?ExceptionEnum.EXCEPTION_MSG_CODE_SEND_ERROR.value():ExceptionEnum.EXCEPTION_MAIL_CODE_SEND_ERROR.value(), VerifyTypeEnum.PHONE.code().equals(type) ?ExceptionEnum.EXCEPTION_MSG_CODE_SEND_ERROR.msg():ExceptionEnum.EXCEPTION_MAIL_CODE_SEND_ERROR.msg(), false);
        }
    }


    /**
     * 实时检查邮件验证码
     * @param type 验证码类型  login  user_register
     * @param account 账号
     * @param code 验证码
     * @return
     */
    @GetMapping("/isEmailCodeValid")
    public ResultMap<Boolean> isEmailCodeValid(@RequestParam String type, @RequestParam String account, @RequestParam String code,@RequestParam String appId) {
        EmailCodeTypeEnum typeEnum = EmailCodeTypeEnum.getEnumByValue(type);
        authService.isEmailCodeValid(typeEnum,account,code,appId);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,Boolean.TRUE);
    }


    private String getUserId(Principal principal) {
        if (principal instanceof UsernamePasswordAuthenticationToken) {
            return ((CustomUserDetails) (((UsernamePasswordAuthenticationToken) principal).getPrincipal())).getId();
        }
        throw new BackendRuntimeException(CommonConstant.EXCEPTION_MSG_LOGIN_USER_NOT_FOUND);
    }

    /**
     * 邮箱验证码登录
     * @return
     */
    @PostMapping("/memberLogin")
    public ResultMap<TokenVO> memberLogin(@RequestBody LoginParam loginParam) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, authService.memberLogin(loginParam));
    }

    /**
     * 获取用户登录设备列表（用于登录时设备超出限制数）
     * @param email 邮箱
     * @param appId appID (PDF Reader Pro 为1)
     * @return List<LoginDevice>
     */
    @PostMapping("/getLoginDeviceList")
    public ResultMap<LoginDeviceMapVO> getLoginDeviceList(@RequestParam String email, @RequestParam String appId) {
        List<LoginDevice> loginDeviceList = authService.getLoginDeviceList(email,appId);
        LoginDeviceMapVO vo = new LoginDeviceMapVO();
        loginDeviceList.forEach(item -> {
            if(!StringUtils.isEmpty(item.getModel())){
                String model = item.getModel().toLowerCase();
                if(model.contains("mac")){
                    model = "Mac";
                }
                if(model.contains("windows")){
                    model = "Windows";
                }
                if(model.contains("ios")){
                    model = "iOS";
                }
                if(model.contains("android")){
                    model = "Android";
                }
                item.setModel(model);
            }
        });
        vo.setDeviceList(loginDeviceList);
        vo.setMaxDeviceNum(authService.getMaxDeviceName(email,appId));
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vo);
    }

    /**
     * 登出设备（用于设备记录）
     * @param deviceId 设备记录Id
     * @return String
     */
    @PostMapping("/logoutDevice")
    public ResultMap<String> logoutDevice(@RequestParam String deviceId) {
        authService.logoutDevice(deviceId);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, CommonConstant.RESULT_SUCCESS);
    }


}
